#include "pgresfunc.h"

/*
 * pgresfunc.c
 *
 * Various functions to make Postgres95 programming more humane
 *
 * Written by: Nathan L. Cutler
 *
 * 199702 taken from file originally written as part of the BSC/dbquery
 * program
 */

/*
 * initBackend()
 *
 * Initializes the database using the database name in the environment 
 * variable MDEBSDB
 *
 * Initialization in this sense means setting up the backend to use this
 * database and connecting to the backend.
 *
 * The function either returns or dies with a fatal error message.
 */
void initBackend()
{
  char *db;

   /* initialize the backend connection parameters */
   pghost = NULL;
   pgport = NULL;
   pgoptions = NULL;
   pgtty = NULL;

   /* get the name of the database from the environment */
   if ((db = getenv("MDEBSDB")) == NULL) {
     fprintf(stderr, "Environment variable MDEBSDB must be defined.\n");
     exit(0);
   }

   /* connect to the Postgres95 backend */
   conn = PQsetdb(pghost, pgport, pgoptions, pgtty, db);

   /* check to see that the backend connection was made successfully */
   if (PQstatus(conn) == CONNECTION_BAD) {
     fprintf(stderr, "initBackend(): Connection to database %s failed.\n",
	     db);
     fprintf(stderr, "%s", PQerrorMessage(conn));
     exit_nicely();
   }

   return;
}

void closeBackend()
{
   PQfinish(conn);
}

int numberReturned()
{
  return PQntuples(res);
}

/*
 * runQuery - runs an arbitrary query, exit_nicely() if query fails to run
 */
int runQuery(char *query)
{
  res = PQexec(conn, query);
  if (res == NULL) {
    fprintf(stderr, "Fatal Error: Query failed: %s", PQerrorMessage(conn));
    fprintf(stderr, "Offending query was: %s\n.", query);
    PQclear(res);
    exit_nicely();
  }
  switch (PQresultStatus(res)) {
  case (PGRES_EMPTY_QUERY): 
    return 1;
  case (PGRES_COMMAND_OK): 
    return 2;
  case (PGRES_TUPLES_OK): 
    return 3;
  case (PGRES_COPY_OUT): 
    return 4;
  case (PGRES_COPY_IN): 
    return 5;
  case (PGRES_BAD_RESPONSE): 
    return 6;
  case (PGRES_NONFATAL_ERROR): 
    return 7;
  case (PGRES_FATAL_ERROR): 
    return 8;
  default: 
    return -1;
  }
}

/* runQueryInTrans - runs an arbitrary query, always return, with
 * various values
 *
 * Useful for running queries in a transaction, when it is necessary to 
 * roll back the transaction if one of the queries fails.  */
int runQueryInTrans(char *query)
{
  res = PQexec(conn, query);
  if (res == NULL) {
    fprintf(stderr, "Fatal Error: Query failed: %s", PQerrorMessage(conn));
    fprintf(stderr, "Offending query was: %s\n.", query);
    PQclear(res);
    return 0;
  }
  switch (PQresultStatus(res)) {
  case (PGRES_EMPTY_QUERY): 
    return 1;
  case (PGRES_COMMAND_OK): 
    return 2;
  case (PGRES_TUPLES_OK): 
    return 3;
  case (PGRES_COPY_OUT): 
    return 4;
  case (PGRES_COPY_IN): 
    return 5;
  case (PGRES_BAD_RESPONSE): 
    return 6;
  case (PGRES_NONFATAL_ERROR): 
    return 7;
  case (PGRES_FATAL_ERROR): 
    return 8;
  default: 
    return -1;
  }
}

void fetchAndDisplay(char *curs)
{
  char query[100];
  sprintf(query, "FETCH ALL in %s;", curs);
  res = PQexec(conn, query);
  if (PQresultStatus(res) != PGRES_TUPLES_OK) {
    fprintf(stderr, "FETCH ALL command didn't return tuples correctly.\n");
    PQclear(res);
    exit_nicely();
  }
  PQdisplayTuples(res, stdout, 1, "|", 1, 0);
  PQclear(res);
  return;
}

/*
 * fetchNext()
 *
 * Given the name of a cursor, fetches the next tuple
 * Returns 1 on successful FETCH, 0 if no more records to fetch
 * Bails out on error.
 */
int fetchNext(char *curs)
{
  char query[100];
  short resultcode;

  sprintf(query, "FETCH 1 IN %s;", curs);
  res = PQexec(conn, query);
  if ((resultcode = PQresultStatus(res)) != PGRES_TUPLES_OK) {
    fprintf(stderr, "FETCH 1 command didn't return tuples correctly.\n");
    PQclear(res);
    exit_nicely();
  }
  return PQntuples(res);
}

void startTransaction()
{
  PGresult* res;

   /* start a transaction block */
   res = PQexec(conn, "BEGIN");
   if (PQresultStatus(res) != PGRES_COMMAND_OK) {
     fprintf(stderr, "BEGIN command failed.\n");
     PQclear(res);
     exit_nicely();
   }
   PQclear(res);

   return;
}

void endTransaction()
{
   PGresult* res;

   /* End the transaction */
   res = PQexec(conn, "END");
   PQclear(res);
}

void rollbackTransaction()
{
   PGresult* res;

   /* End the transaction */
   res = PQexec(conn, "ABORT");
   PQclear(res);
}

void exit_nicely()
{
  PQfinish(conn);
  exit(1);
}

/*
 * getField()
 *
 * ASSUMES THAT res POINTS TO ONLY ONE TUPLE
 */
char *getField(char *fieldDesig)
{
  char *returnedDesig;
  char fieldIndex;

  fieldIndex = PQfnumber(res, fieldDesig);
  if (fieldIndex == -1) {
    fprintf(stderr, "Fatal error: Nonexistent field \"%s\".\n", fieldDesig);
    exit_nicely();
  }
  /*  printf("Field number is %d.\n", fieldIndex); */
  returnedDesig = PQgetvalue(res, 0, fieldIndex);
  return returnedDesig;
}
